# Copyright (c) Nuralogix. All rights reserved. Licensed under the MIT license.
# See LICENSE.txt in the project root for license information

import json
from typing import Any, Tuple, Union

import aiohttp

from .Settings import Settings


class Base:
    @classmethod
    async def _get(cls, session: aiohttp.ClientSession, url_fragment: str, params: dict = None, **kwargs: Any) -> Any:
        url = f"{Settings.rest_url}/{url_fragment}"

        async with session.get(url, params=params, **kwargs) as resp:
            if resp.content_type != "application/json":
                return resp.status, await resp.read()
            else:
                return resp.status, await resp.json()

    @classmethod
    async def _post(cls, session: aiohttp.ClientSession, url_fragment: str, data: Union[dict, list],
                    **kwargs: Any) -> Any:
        url = f"{Settings.rest_url}/{url_fragment}"

        async with session.post(url, json=data, **kwargs) as resp:
            return resp.status, await resp.json()

    @classmethod
    async def _patch(cls, session: aiohttp.ClientSession, url_fragment: str, data: dict, **kwargs: Any) -> Any:
        url = f"{Settings.rest_url}/{url_fragment}"

        async with session.patch(url, json=data, **kwargs) as resp:
            return resp.status, await resp.json()

    @classmethod
    async def _delete(cls,
                      session: aiohttp.ClientSession,
                      url_fragment: str,
                      data: Union[dict, list] = None,
                      **kwargs: Any) -> Any:
        url = f"{Settings.rest_url}/{url_fragment}"

        async with session.delete(url, json=data, **kwargs) as resp:
            return resp.status, await resp.json()

    @classmethod
    def ws_decode(cls, msg: aiohttp.WSMessage) -> Tuple[int, str, bytes]:
        if msg.type != aiohttp.WSMsgType.BINARY:
            raise ValueError("WebSocket error: Expecting only binary websocket responses")
        result = msg.data
        request_id = result[:10].decode('utf-8')
        status = int(result[10:13].decode('utf-8'))
        payload = result[13:].decode('utf-8')

        # TODO: Handle raise_for_status == False
        if status >= 400:
            try:
                error = json.loads(payload)
            except Exception as e:
                raise ValueError(
                    f"WebSocket error: API Status {status} for req#:{request_id}, could not parse response as JSON"
                ) from e
            raise ValueError(f"WebSocket error: API Status {status} for req#:{request_id}, Code: {error['Code']}, "
                             f"Description: '{error['Errors'] if 'Errors' in error else None}'.")

        return status, request_id, payload

    @classmethod
    def ws_connect(cls, session: aiohttp.ClientSession, **kwargs: Any):
        return session.ws_connect(Settings.ws_url, protocols=["json"], **kwargs)



# Copyright (c) Nuralogix. All rights reserved. Licensed under the MIT license.
# See LICENSE.txt in the project root for license information

import base64
import json
import warnings
from typing import Any, Union, Optional

import aiohttp

from .Base import Base


class Measurements(Base):
    url_fragment = "measurements"

    @classmethod
    async def create(cls,
                     session: aiohttp.ClientSession,
                     study_id: str,
                     resolution: int = 0,
                     user_profile_id: str = "",
                     partner_id: str = "",
                     **kwargs: Any) -> Any:
        data = {
            "StudyID": study_id,
            "Resolution": resolution,
            "UserProfileID": user_profile_id,
            "PartnerID": partner_id,
        }

        return await cls._post(session, cls.url_fragment, data=data, **kwargs)

    @classmethod
    async def add_data(cls,
                       session: aiohttp.ClientSession,
                       measurement_id: str,
                       action: str,
                       payload: Union[bytes, bytearray, memoryview],
                       *,
                       chunk_order: Optional[Union[str, int]] = None,
                       start_time_s: Optional[str] = None,
                       end_time_s: Optional[str] = None,
                       duration_s: Optional[str] = None,
                       metadata: Optional[Union[bytes, bytearray, memoryview]] = None,
                       **kwargs: Any) -> Any:
        data = {
            "Action": action,
            "Payload": base64.standard_b64encode(payload).decode('ascii'),
        }
        data = {k: v for k, v in data.items() if v is not None}

        return await cls._post(session, f"{cls.url_fragment}/{measurement_id}/data", data=data, **kwargs)

    @classmethod
    async def list(cls,
                   session: aiohttp.ClientSession,
                   date: str = "",
                   end_date: str = "",
                   user_profile_id: str = "",
                   user_profile_name: str = "",
                   study_id: str = "",
                   status_id: str = "",
                   partner_id: str = "",
                   mode: str = "",
                   limit: int = 50,
                   offset: int = 0,
                   **kwargs: Any) -> Any:
        params = {
            "Date": date,
            "EndDate": end_date,
            "UserProfileID": user_profile_id,
            "UserProfileName": user_profile_name,
            "StudyID": study_id,
            "StatusID": status_id.upper(),
            "PartnerID": partner_id,
            "Mode": mode.upper(),
            "Limit": limit,
            "Offset": offset,
        }

        return await cls._get(session, cls.url_fragment, params=params, **kwargs)

    @classmethod
    async def retrieve(cls,
                       session: aiohttp.ClientSession,
                       measurement_id: str,
                       expand: bool = True,
                       **kwargs: Any) -> Any:
        params = {}
        if expand:
            params["ExpandResults"] = "true"
        return await cls._get(session, f"{cls.url_fragment}/{measurement_id}", params=params, **kwargs)

    @classmethod
    async def ws_subscribe_to_results(cls, ws: aiohttp.ClientWebSocketResponse, request_id: Union[str, int],
                                      measurement_id: str, results_request_id: Union[str, int]) -> None:
        action_id = "0510"

        request = {
            "Params": {
                "ID": measurement_id,
            },
            "RequestID": str(results_request_id),
        }

        ws_request = f"{action_id:4}{request_id:10}{json.dumps(request)}"

        await ws.send_str(ws_request)

    @classmethod
    async def ws_add_data(
        cls,
        ws: aiohttp.ClientWebSocketResponse,
        request_id: str,
        measurement_id: str,
        action: str,
        payload: Union[bytes, bytearray, memoryview],
        *,
        chunk_order: Optional[Union[str, int]] = None,
        start_time_s: Optional[str] = None,
        end_time_s: Optional[str] = None,
        duration_s: Optional[str] = None,
        metadata: Optional[Union[bytes, bytearray, memoryview]] = None,
    ) -> None:
        action_id = "0506"

        request = {
            "Params": {
                "ID": measurement_id,
            },
            "ChunkOrder": int(chunk_order) if chunk_order is not None else None,
            "Action": action,
            "StartTime": start_time_s,
            "EndTime": end_time_s,
            "Duration": int(duration_s) if duration_s is not None else None,
            "Meta": base64.standard_b64encode(metadata).decode('ascii') if metadata else None,
            "Payload": base64.standard_b64encode(payload).decode('ascii'),
        }
        request = {k: v for k, v in request.items() if v is not None}

        ws_request = f"{action_id:4}{request_id:10}{json.dumps(request)}"

        await ws.send_str(ws_request)

    @classmethod
    async def delete(cls, session: aiohttp.ClientSession, measurement_id: str, **kwargs: Any) -> Any:
        warnings.warn(f"{cls.delete.__qualname__} is deprecated and will be removed.", DeprecationWarning)

        return await cls._delete(session, f"{cls.url_fragment}/{measurement_id}", **kwargs)

    @classmethod
    async def retrieve_intermediate(cls,
                       session: aiohttp.ClientSession,
                       measurement_id: str,
                       chunk_order: int,
                       **kwargs: Any) -> Any:
        return await cls._get(session, f"{cls.url_fragment}/{measurement_id}/results/{chunk_order}", **kwargs)



# Copyright (c) Nuralogix. All rights reserved. Licensed under the MIT license.
# See LICENSE.txt in the project root for license information

from typing import Any, Optional

import aiohttp

from .Base import Base
from .Settings import Settings


class Auths(Base):
    url_fragment = "auths"

    @classmethod
    async def request_password_reset_email(cls, session: aiohttp.ClientSession, email: str, org_id: str,
                                           **kwargs: Any) -> Any:
        data = {
            "Email": email,
            "Identifier": org_id,
        }
        return await cls._patch(session, f"{cls.url_fragment}/users/reset", data=data, **kwargs)

    @classmethod
    async def request_login_code(cls, session: aiohttp.ClientSession, phone_number: str, org_id: str,
                                 **kwargs: Any) -> Any:
        data = {
            "PhoneNumber": phone_number,
            "Identifier": org_id,
        }
        return await cls._patch(session, f"{cls.url_fragment}/users/code", data=data, **kwargs)

    @classmethod
    async def renew_token(cls,
                          session: aiohttp.ClientSession,
                          token: str,
                          refresh_token: str,
                          refresh_token_expires_sec: Optional[int] = None,
                          **kwargs: Any) -> Any:
        data = {
            "Token": token,
            "RefreshToken": refresh_token,
            "RefreshTokenExpiresIn": int(refresh_token_expires_sec) if refresh_token_expires_sec is not None else None
        }
        return await cls._post(session, f"{cls.url_fragment}/renew", data=data, **kwargs)

    @classmethod
    async def renew_user_token(cls,
                               session: aiohttp.ClientSession,
                               refresh_token_expires_sec: Optional[int] = None,
                               **kwargs: Any) -> Any:
        status, body = await cls.renew_token(session, Settings.user_token, Settings.user_refresh_token,
                                             refresh_token_expires_sec, **kwargs)

        if status < 400:
            Settings.user_token = body["Token"]
            Settings.user_refresh_token = body["RefreshToken"]

        return status, body

    @classmethod
    async def renew_device_token(cls,
                                 session: aiohttp.ClientSession,
                                 refresh_token_expires_sec: Optional[int] = None,
                                 **kwargs: Any) -> Any:
        status, body = await cls.renew_token(session, Settings.device_token, Settings.device_refresh_token,
                                             refresh_token_expires_sec, **kwargs)

        if status < 400:
            Settings.device_token = body["Token"]
            Settings.device_refresh_token = body["RefreshToken"]

        return status, body

    @classmethod
    async def request_child_token(cls,
                                  session: aiohttp.ClientSession,
                                  token: str,
                                  refresh_token: str,
                                  token_expires_sec: Optional[int] = None,
                                  **kwargs: Any) -> Any:
        data = {
            "Token": token,
            "RefreshToken": refresh_token,
            "expiryInSec": int(token_expires_sec) if token_expires_sec is not None else None,
        }
        return await cls._post(session, f"{cls.url_fragment}/generateToken", data=data, **kwargs)



# Copyright (c) Nuralogix. All rights reserved. Licensed under the MIT license.
# See LICENSE.txt in the project root for license information

from typing import Any

import aiohttp

from .Base import Base


class Licenses(Base):
    url_fragment = "licenses"

    @classmethod
    async def list(cls,
                   session: aiohttp.ClientSession,
                   date: str = "",
                   end_date: str = "",
                   status_id: str = "",
                   license_type_id: str = "",
                   license_type: str = "",
                   limit: int = 25,
                   offset: int = 0,
                   **kwargs: Any) -> Any:
        params = {
            "StatusID": status_id.upper(),
            "LicenseTypeID": license_type_id,
            "LicenseType": license_type,
            "Date": date,
            "EndDate": end_date,
            "Limit": limit,
            "Offset": offset,
        }
        return await cls._get(session, f"{cls.url_fragment}/organization", params=params, **kwargs)

    @classmethod
    async def get(cls, session: aiohttp.ClientSession, license_id: str = "", **kwargs: Any) -> Any:
        return await cls._get(session, f"{cls.url_fragment}/organization/{license_id}", **kwargs)



# Copyright (c) Nuralogix. All rights reserved. Licensed under the MIT license.
# See LICENSE.txt in the project root for license information

from .Auths import Auths
from .Devices import Devices
from .General import General
from .Licenses import Licenses
from .Measurements import Measurements
from .Organizations import Organizations
from .Profiles import Profiles
from .Settings import Settings
from .Studies import Studies
from .Users import Users



# Copyright (c) Nuralogix. All rights reserved. Licensed under the MIT license.
# See LICENSE.txt in the project root for license information

from typing import Any

import aiohttp

from .Base import Base


class General(Base):
    @classmethod
    async def api_status(cls, session: aiohttp.ClientSession, **kwargs: Any) -> Any:
        return await cls._get(session, "status", **kwargs)

    @classmethod
    async def list_available_statuses(cls, session: aiohttp.ClientSession, **kwargs: Any) -> Any:
        return await cls._get(session, "statuses", **kwargs)

    @classmethod
    async def list_available_user_roles(cls, session: aiohttp.ClientSession, **kwargs: Any) -> Any:
        return await cls._get(session, "roles", **kwargs)

    @classmethod
    async def list_accepted_mime_types(cls, session: aiohttp.ClientSession, **kwargs: Any) -> Any:
        return await cls._get(session, "mimes", **kwargs)

    @classmethod
    async def verify_token(cls, session: aiohttp.ClientSession, **kwargs: Any) -> Any:
        return await cls._get(session, "auth", **kwargs)

    @classmethod
    async def list_available_regions(cls, session: aiohttp.ClientSession, **kwargs: Any) -> Any:
        return await cls._get(session, "regions", **kwargs)



# Copyright (c) Nuralogix. All rights reserved. Licensed under the MIT license.
# See LICENSE.txt in the project root for license information

from typing import Any

import aiohttp

from .Base import Base


class Profiles(Base):
    url_fragment = "users/profiles"

    @classmethod
    async def create(cls, session: aiohttp.ClientSession, profile_name: str, email: str, **kwargs: Any) -> Any:
        data = {
            "Name": profile_name,
            "Email": email,
        }

        status, body = await cls._post(session, cls.url_fragment, data=data, **kwargs)

        return status, body

    @classmethod
    async def update(cls, session: aiohttp.ClientSession, profile_id: str, profile_name: str, profile_email: str,
                     status: str, **kwargs: Any) -> Any:
        data = {
            "Name": profile_name,
            "Email": profile_email,
            "Status": status,
        }
        return await cls._patch(session, f"{cls.url_fragment}/{profile_id}", data=data, **kwargs)

    @classmethod
    async def delete(cls, session: aiohttp.ClientSession, profile_id: str, **kwargs: Any) -> Any:
        return await cls._delete(session, f"{cls.url_fragment}/{profile_id}", **kwargs)

    @classmethod
    async def retrieve(cls, session: aiohttp.ClientSession, profile_id: str, **kwargs: Any) -> Any:
        return await cls._get(session, f"{cls.url_fragment}/{profile_id}", **kwargs)

    @classmethod
    async def list(cls,
                   session: aiohttp.ClientSession,
                   profile_name: str = "",
                   status: str = "",
                   limit: int = 25,
                   offset: int = 0,
                   **kwargs: Any) -> Any:
        params = {
            "UserProfileName": profile_name,
            "Status": status,
            "Limit": limit,
            "Offset": offset,
        }

        return await cls._get(session, cls.url_fragment, params=params, **kwargs)

    @classmethod
    async def list_by_user(cls,
                           session: aiohttp.ClientSession,
                           user_id: str = "",
                           profile_name: str = "",
                           status: str = "",
                           limit: int = 25,
                           offset: int = 0,
                           **kwargs: Any) -> Any:
        params = {
            "UserProfileName": profile_name,
            "Status": status,
            "Limit": limit,
            "Offset": offset,
        }

        return await cls._get(session, f"users/{user_id}/profiles", params=params, **kwargs)



# Copyright (c) Nuralogix. All rights reserved. Licensed under the MIT license.
# See LICENSE.txt in the project root for license information

import warnings
from typing import Any

import aiohttp

from .Base import Base


class Devices(Base):
    url_fragment = "devices"

    @classmethod
    async def types(cls, session: aiohttp.ClientSession, **kwargs: Any) -> Any:
        return await cls._get(session, f"{cls.url_fragment}/types", **kwargs)

    @classmethod
    async def create(cls, session: aiohttp.ClientSession, device_name: str, device_type_id: str, device_id: str,
                     version: str, **kwargs: Any) -> Any:
        data = {
            "Name": device_name,
            "DeviceTypeID": device_type_id,
            "Identifier": device_id,  # TODO: Describe parameter
            "Version": version,
        }

        return await cls._post(session, cls.url_fragment, data=data, **kwargs)

    @classmethod
    async def update(cls,
                     session: aiohttp.ClientSession,
                     id: str,
                     device_name: str = "",
                     device_type_id: str = "",
                     device_id: str = "",
                     status: str = "",
                     version: str = "",
                     **kwargs: Any) -> Any:
        data = {
            "Name": device_name,
            "DeviceTypeID": device_type_id,
            "Status": status,
            "Identifier": device_id,  # TODO: Describe parameter
            "Version": version,
        }

        return await cls._patch(session, f"{cls.url_fragment}/{id}", data=data, **kwargs)

    @classmethod
    async def retrieve(cls, session: aiohttp.ClientSession, device_id: str, **kwargs: Any) -> Any:
        return await cls._get(session, f"{cls.url_fragment}/{device_id}", **kwargs)

    @classmethod
    async def list(cls,
                   session: aiohttp.ClientSession,
                   status_id: str = "",
                   device_type_id: str = "",
                   device_name: str = "",
                   device_version: str = "",
                   license_id: str = "",
                   date: str = "",
                   end_date: str = "",
                   sort_order: str = "",
                   limit: int = 25,
                   offset: int = 0,
                   **kwargs: Any) -> Any:
        params = {
            "StatusID": status_id,
            "DeviceTypeID": device_type_id,
            "LicenseID": license_id,
            "Name": device_name,  # TODO: Verify docs
            "Version": device_version,
            "Date": date,
            "EndDate": end_date,
            "SortOrder": sort_order.upper(),
            "Limit": limit,
            "Offset": offset,
        }

        return await cls._get(session, cls.url_fragment, params=params, **kwargs)

    @classmethod
    async def delete(cls, session: aiohttp.ClientSession, device_id: str, **kwargs: Any) -> Any:
        warnings.warn(f"{cls.delete.__qualname__} is deprecated and will be removed.", DeprecationWarning)

        return await cls._delete(session, f"{cls.url_fragment}/{device_id}", **kwargs)

    @classmethod
    async def delete_measurements(cls, session: aiohttp.ClientSession, device_id: str, **kwargs: Any) -> Any:
        warnings.warn(f"{cls.delete_measurements.__qualname__} is deprecated and will be removed.", DeprecationWarning)

        return await cls._delete(session, f"{cls.url_fragment}/{device_id}/measurements", **kwargs)

    @classmethod
    async def retrieve_license_id(cls, session: aiohttp.ClientSession, **kwargs: Any) -> Any:
        return await cls._get(session, f"{cls.url_fragment}/license", **kwargs)



# Copyright (c) Nuralogix. All rights reserved. Licensed under the MIT license.
# See LICENSE.txt in the project root for license information

import warnings
from typing import Any, Optional

import aiohttp

from .Base import Base


class Studies(Base):
    url_fragment = "studies"

    @classmethod
    async def types(cls, session: aiohttp.ClientSession, status: str, **kwargs: Any) -> Any:
        params = {
            "StatusID": status,
        }
        return await cls._get(session, f"{cls.url_fragment}/types", params=params, **kwargs)

    @classmethod
    async def list_templates(cls,
                             session: aiohttp.ClientSession,
                             status: str,
                             type_: str,
                             sort_order: str = "",
                             sort_key: str = "",
                             limit: int = 25,
                             offset: int = 0,
                             **kwargs: Any) -> Any:
        params = {
            "Status": status,
            "Type": type_,
            "SortOrder": sort_order.upper(),
            "SortBy": sort_key,
            "Limit": limit,
            "Offset": offset,
        }
        return await cls._get(session, f"{cls.url_fragment}/templates", params=params, **kwargs)

    @classmethod
    async def create(cls, session: aiohttp.ClientSession, study_name: str, description: str, study_template_id: str,
                     config: dict, **kwargs: Any) -> Any:
        data = {
            "Name": study_name,
            "Description": description,
            "StudyTemplateID": study_template_id,
            "Config": config,
        }

        return await cls._post(session, cls.url_fragment, data=data, **kwargs)

    @classmethod
    async def update(cls,
                     session: aiohttp.ClientSession,
                     study_id: str,
                     study_name: str = "",
                     description: str = "",
                     status: str = "",
                     config: Optional[dict] = None,
                     **kwargs: Any) -> Any:
        data = {
            "Name": study_name,
            "StatusID": status.upper(),
            "Description": description,
            "Config": config,
        }

        return await cls._patch(session, f"{cls.url_fragment}/{study_id}", data=data, **kwargs)

    @classmethod
    async def retrieve(cls, session: aiohttp.ClientSession, study_id: str, **kwargs: Any) -> Any:
        return await cls._get(session, f"{cls.url_fragment}/{study_id}", **kwargs)

    @classmethod
    async def delete(cls, session: aiohttp.ClientSession, study_id: str, **kwargs: Any) -> Any:
        return await cls._delete(session, f"{cls.url_fragment}/{study_id}", **kwargs)

    @classmethod
    async def list(cls,
                   session: aiohttp.ClientSession,
                   date: str = "",
                   end_date: str = "",
                   study_name: str = "",
                   status: str = "",
                   limit: int = 25,
                   offset: int = 0,
                   **kwargs: Any) -> Any:
        params = {
            "Date": date,
            "EndDate": end_date,
            "Name": study_name,
            "StatusID": status.upper(),
            "Limit": limit,
            "Offset": offset,
        }

        return await cls._get(session, cls.url_fragment, params=params, **kwargs)

    @classmethod
    async def retrieve_sdk_config_data(cls, session: aiohttp.ClientSession, study_id: str, sdk_id: str,
                                       current_hash: str, **kwargs: Any) -> Any:
        data = {
            "StudyID": study_id,
            "SDKID": sdk_id,
            "MD5Hash": current_hash,
        }

        return await cls._post(session, f"{cls.url_fragment}/sdkconfig", data=data, **kwargs)

    @classmethod
    async def delete_study_measurements(cls, session: aiohttp.ClientSession, study_id: str, **kwargs: Any) -> Any:
        warnings.warn(f"{cls.delete_study_measurements.__qualname__} is deprecated and will be removed.",
                      DeprecationWarning)

        return await cls._delete(session, f"{cls.url_fragment}/{study_id}/measurements", **kwargs)



# Copyright (c) Nuralogix. All rights reserved. Licensed under the MIT license.
# See LICENSE.txt in the project root for license information

import base64
import json
import warnings
from typing import Any, Union

import aiohttp

from .Base import Base
from .Settings import Settings


class Organizations(Base):
    url_fragment = "organizations"

    @classmethod
    async def retrieve(cls, session: aiohttp.ClientSession, **kwargs: Any) -> Any:
        return await cls._get(session, cls.url_fragment, **kwargs)

    @classmethod
    async def update(cls,
                     session: aiohttp.ClientSession,
                     org_name: str = "",
                     org_id: str = "",
                     contact_name: str = "",
                     contact_email: str = "",
                     logo: Union[None, bytes, bytearray, memoryview] = None,
                     **kwargs: Any) -> Any:
        data = {
            "Name": org_name,
            "Identifier": org_id,
            "Contact": contact_name,
            "Email": contact_email,
            "Logo": base64.standard_b64encode(logo).decode('ascii') if logo is not None else logo,
        }
        return await cls._patch(session, cls.url_fragment, data=data, **kwargs)

    @classmethod
    async def list_users(cls,
                         session: aiohttp.ClientSession,
                         start_date: str = "",
                         end_date: str = "",
                         email: str = "",
                         role_id: str = "",
                         gender: str = "",
                         region: str = "",
                         limit: int = 25,
                         offset: int = 0,
                         **kwargs: Any) -> Any:
        params = {
            "Date": start_date,
            "EndDate": end_date,
            "Username": email,
            "RoleID": role_id,
            "Gender": gender,
            "Region": region.lower(),
            "Limit": limit,
            "Offset": offset,
        }

        return await cls._get(session, f"{cls.url_fragment}/users", params=params, **kwargs)

    @classmethod
    async def create_user(cls,
                          session: aiohttp.ClientSession,
                          first_name: str,
                          last_name: str,
                          email: str,
                          gender: str,
                          date_of_birth: str,
                          role_id: str,
                          **kwargs: Any) -> Any:
        data = {
            "FirstName": first_name,
            "LastName": last_name,
            "Email": email,
            "Gender": gender,
            "DateOfBirth": date_of_birth,
            "RoleID": role_id
        }

        return await cls._post(session, f"{cls.url_fragment}/users", data=data, **kwargs)

    @classmethod
    async def register_license(cls,
                               session: aiohttp.ClientSession,
                               license_key: str,
                               device_type_id: str,
                               app_name: str,
                               app_id: str,
                               app_version: str,
                               token_expires_in_seconds: int = 0,
                               token_subject: str = "",
                               refresh_token_expires_in_sec: int = 0,
                               **kwargs: Any) -> Any:
        data = {
            "Key": license_key,
            "DeviceTypeID": device_type_id,  # TODO: Describe list of allowed values here and in params below
            "Name": app_name,
            "Identifier": app_id,
            "Version": app_version[:20],
            "TokenSubject": token_subject,
            "TokenExpiresIn": token_expires_in_seconds if token_expires_in_seconds > 0 else 86400,
            "RefreshTokenExpiresIn": refresh_token_expires_in_sec if refresh_token_expires_in_sec > 0 else 2592000
        }

        status, body = await cls._post(session, f"{cls.url_fragment}/licenses", data=data, **kwargs)

        if status < 400:
            Settings.device_id = body["DeviceID"]
            Settings.device_token = body["Token"]
            Settings.device_refresh_token = body["RefreshToken"]
            Settings.role_id = body["RoleID"]
            Settings.user_id = body["UserID"]  # TODO: Verify why returned

        return status, body

    @classmethod
    async def unregister_license(cls, session: aiohttp.ClientSession, **kwargs: Any) -> Any:
        status, body = await cls._delete(session, f"{cls.url_fragment}/licenses", **kwargs)

        if status < 400:
            Settings.device_token = ""
            Settings.device_id = ""
            Settings.role_id = ""
            Settings.user_id = ""
            Settings.device_refresh_token = ""

        return status, body

    @classmethod
    async def retrieve_logo(cls, session: aiohttp.ClientSession, org_id: str, **kwargs: Any) -> Any:
        return await cls._get(session, f"{cls.url_fragment}/{org_id}/logo", **kwargs)

    @classmethod
    async def list_measurements(cls,
                                session: aiohttp.ClientSession,
                                date: str = "",
                                end_date: str = "",
                                user_profile_id: str = "",
                                user_profile_name: str = "",
                                study_id: str = "",
                                status_id: str = "",
                                email: str = "",
                                partner_id: str = "",
                                mode: str = "",
                                region: str = "",
                                limit: int = 50,
                                offset: int = 0,
                                **kwargs: Any) -> Any:
        params = {
            "Date": date,
            "EndDate": end_date,
            "UserProfileID": user_profile_id,
            "UserProfileName": user_profile_name,
            "StudyID": study_id,
            "StatusID": status_id,
            "UserName": email,
            "PartnerID": partner_id,
            "Mode": mode.upper(),
            "Region": region.lower(),
            "Limit": limit,
            "Offset": offset,
        }

        return await cls._get(session, f"{cls.url_fragment}/measurements", params=params, **kwargs)

    @classmethod
    async def retrieve_measurement(cls,
                                   session: aiohttp.ClientSession,
                                   measurement_id: str,
                                   expand: bool = True,
                                   **kwargs: Any) -> Any:
        params = {}
        if expand:
            params["ExpandResults"] = "true"
        return await cls._get(session, f"{cls.url_fragment}/measurements/{measurement_id}", params=params, **kwargs)

    @classmethod
    async def list_profiles(
        cls,
        session: aiohttp.ClientSession,
        date: str = "",
        end_date: str = "",
        owner_email: str = "",
        user_profile_name: str = "",
        status_id: str = "",
        limit: int = 25,
        offset: int = 0,
        **kwargs: Any,
    ) -> Any:
        params = {
            "Date": date,
            "EndDate": end_date,
            "OwnerUser": owner_email,
            "UserProfileName": user_profile_name,
            "StatusID": status_id,
            "Limit": limit,
            "Offset": offset,
        }

        return await cls._get(session, f"{cls.url_fragment}/profiles", params=params, **kwargs)

    @classmethod
    async def retrieve_profile(cls, session: aiohttp.ClientSession, profile_id: str, **kwargs: Any) -> Any:
        return await cls._get(session, f"{cls.url_fragment}/profiles/{profile_id}", **kwargs)

    @classmethod
    async def update_profile(cls,
                             session: aiohttp.ClientSession,
                             profile_id: str,
                             name: str = "",
                             email: str = "",
                             status: str = "",
                             **kwargs: Any) -> Any:
        data = {
            "Name": name,
            "Email": email,
            "Status": status,
        }

        return await cls._patch(session, f"{cls.url_fragment}/profiles/{profile_id}", data=data, **kwargs)

    @classmethod
    async def retrieve_user(cls, session: aiohttp.ClientSession, user_id: str, **kwargs: Any) -> Any:
        return await cls._get(session, f"{cls.url_fragment}/users/{user_id}", **kwargs)

    @classmethod
    async def update_user(cls,
                          session: aiohttp.ClientSession,
                          user_id: str,
                          first_name: str = "",
                          last_name: str = "",
                          gender: str = "",
                          date_of_birth: str = "",
                          height_cm: Union[str, int] = "",
                          weight_kg: Union[str, int] = "",
                          **kwargs: Any) -> Any:
        data = {
            "FirstName": first_name,
            "LastName": last_name,
            "Gender": gender,
            "DateOfBirth": date_of_birth,
            "HeightCm": str(height_cm),
            "WeightKg": str(weight_kg),
        }

        return await cls._patch(session, f"{cls.url_fragment}/users/{user_id}", data=data, **kwargs)

    @classmethod
    async def remove_user(cls, session: aiohttp.ClientSession, user_id: str, **kwargs: Any) -> Any:
        return await cls._delete(session, f"{cls.url_fragment}/users/{user_id}", **kwargs)

    @classmethod
    async def login(cls,
                    session: aiohttp.ClientSession,
                    email: str,
                    password: str,
                    org_id: str,
                    mfa_token: str = "",
                    token_expires_in_sec: int = 0,
                    refresh_token_expires_in_sec: int = 0,
                    **kwargs: Any) -> Any:
        data = {
            "Email": email,
            "Password": password,
            "Identifier": org_id,
            "MFAToken": mfa_token,
            "TokenExpiresIn": token_expires_in_sec if token_expires_in_sec > 0 else 86400,
            "RefreshTokenExpiresIn": refresh_token_expires_in_sec if refresh_token_expires_in_sec > 0 else 2592000
        }

        status, body = await cls._post(session, f"{cls.url_fragment}/auth", data=data, **kwargs)

        if status < 400:
            Settings.user_token = body["Token"]
            Settings.user_refresh_token = body["RefreshToken"]

        return status, body

    @classmethod
    async def ws_auth_with_token(cls, ws: aiohttp.ClientWebSocketResponse, request_id: Union[str, int]) -> None:
        action_id = "0718"

        request = {
            "Token":  Settings.user_token if Settings.user_token else Settings.device_token,
        }

        ws_request = f"{action_id:4}{request_id:10}{json.dumps(request)}"

        await ws.send_str(ws_request)

    @classmethod
    async def delete_all_measurements(cls, session: aiohttp.ClientSession, org_id: str, **kwargs: Any) -> Any:
        warnings.warn(f"{cls.delete_all_measurements.__qualname__} is deprecated and will be removed.",
                      DeprecationWarning)

        return await cls._delete(session, f"{cls.url_fragment}/{org_id}/measurements", **kwargs)

    @classmethod
    async def delete_measurements_by_partnerid(cls, session: aiohttp.ClientSession, org_id: str, partner_id: str,
                                               **kwargs: Any) -> Any:
        warnings.warn(f"{cls.delete_measurements_by_partnerid.__qualname__} is deprecated and will be removed.",
                      DeprecationWarning)

        return await cls._delete(session, f"{cls.url_fragment}/{org_id}/partners/{partner_id}/measurements", **kwargs)



# Copyright (c) Nuralogix. All rights reserved. Licensed under the MIT license.
# See LICENSE.txt in the project root for license information
import pathlib

from setuptools import setup

long_description = (pathlib.Path(__file__).parent / "README.md").read_text()

setup(
    name='dfx-apiv2-client',
    author="NuraLogix Corporation",
    url='https://github.com/nuralogix/dfx-apiv2-client-py',
    version='0.15.0',
    packages=['dfx_apiv2_client'],
    install_requires=[
        'aiohttp<4',
    ],
    description='dfx-apiv2-client is a Python 3 asyncio client library for the Nuralogix DeepAffex API.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    license_file='LICENSE.txt',
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Intended Audience :: Developers",
    ],
)



# Copyright (c) Nuralogix. All rights reserved. Licensed under the MIT license.
# See LICENSE.txt in the project root for license information

import warnings
from typing import Any, Union

import aiohttp

from .Auths import Auths  # Just for __qualname__
from .Base import Base
from .Settings import Settings


class Users(Base):
    url_fragment = "users"

    @classmethod
    async def create(cls,
                     session: aiohttp.ClientSession,
                     first_name: str,
                     last_name: str,
                     email: str,
                     password: str,
                     phone_number: str,
                     gender: str,
                     date_of_birth: str,
                     height_cm: Union[str, int],
                     weight_kg: Union[str, int],
                     **kwargs: Any) -> Any:
        data = {
            "FirstName": first_name,
            "LastName": last_name,
            "Email": email,
            "Password": password,
            "PhoneNumber": phone_number,
            "Gender": gender,
            "DateOfBirth": date_of_birth,
            "HeightCm": str(height_cm),
            "WeightKg": str(weight_kg),
        }

        return await cls._post(session, cls.url_fragment, data=data, **kwargs)

    @classmethod
    async def login(cls,
                    session: aiohttp.ClientSession,
                    email: str,
                    password: str,
                    mfa_token: str = "",
                    token_expires_in_sec: int = 0,
                    refresh_token_expires_in_sec: int = 0,
                    **kwargs: Any) -> Any:
        data = {
            "Email": email,
            "Password": password,
            "MFAToken": mfa_token,
            "TokenExpiresIn": token_expires_in_sec if token_expires_in_sec > 0 else 86400,
            "RefreshTokenExpiresIn": refresh_token_expires_in_sec if refresh_token_expires_in_sec > 0 else 2592000
        }

        status, body = await cls._post(session, f"{cls.url_fragment}/auth", data=data, **kwargs)

        if status < 400:
            Settings.user_token = body["Token"]
            Settings.user_refresh_token = body["RefreshToken"]

        return status, body

    @classmethod
    async def request_phone_login_code(cls, session: aiohttp.ClientSession, org_key: str, phone_number: str,
                                       **kwargs: Any) -> Any:
        warnings.warn(
            f"{cls.request_phone_login_code.__qualname__} is deprecated. "
            f"Please use {Auths.request_login_code.__qualname__} instead.", DeprecationWarning)
        return await cls._get(session, f"{cls.url_fragment}/auth/code/{org_key}/{phone_number}", **kwargs)

    @classmethod
    async def login_with_phone_code(cls,
                                    session: aiohttp.ClientSession,
                                    org_key: str,
                                    phone_number: str,
                                    login_code: str,
                                    token_expires_in_sec: int = 0,
                                    refresh_token_expires_in_sec: int = 0,
                                    **kwargs: Any) -> Any:
        data = {
            "LoginCode": login_code,
            "PhoneNumber": phone_number,
            "OrgKey": org_key,
            "TokenExpiresIn": token_expires_in_sec if token_expires_in_sec > 0 else 86400,
            "RefreshTokenExpiresIn": refresh_token_expires_in_sec if refresh_token_expires_in_sec > 0 else 2592000
        }

        status, body = await cls._post(session, f"{cls.url_fragment}/auth/code", data=data, **kwargs)

        if status < 400:
            Settings.user_token = body["Token"]
            Settings.user_refresh_token = body["RefreshToken"]

        return body

    @classmethod
    async def retrieve(cls, session: aiohttp.ClientSession, **kwargs: Any) -> Any:
        return await cls._get(session, cls.url_fragment, **kwargs)

    @classmethod
    async def update(cls,
                     session: aiohttp.ClientSession,
                     first_name: str,
                     last_name: str,
                     email: str,
                     password: str,
                     phone_number: str,
                     gender: str,
                     date_of_birth: str,
                     height_cm: Union[str, int],
                     weight_kg: Union[str, int],
                     **kwargs: Any) -> Any:
        data = {
            "FirstName": first_name,
            "LastName": last_name,
            "Email": email,
            "Password": password,
            "PhoneNumber": phone_number,
            "Gender": gender,
            "DateOfBirth": date_of_birth,
            "HeightCm": str(height_cm),
            "WeightKg": str(weight_kg),
        }

        return await cls._patch(session, cls.url_fragment, data=data, **kwargs)

    @classmethod
    async def create_mfa_secret(cls, session: aiohttp.ClientSession, **kwargs: Any) -> Any:
        return await cls._post(session, f"{cls.url_fragment}/mfa/secret", **kwargs)

    @classmethod
    async def enable_mfa(cls, session: aiohttp.ClientSession, mfa_secret: str, mfa_token: str, **kwargs: Any) -> Any:
        data = {
            "MFASecret": mfa_secret,
            "MFAToken": mfa_token,
        }

        return await cls._post(session, f"{cls.url_fragment}/mfa", data=data, **kwargs)

    @classmethod
    async def disable_mfa_by_userid(cls, session: aiohttp.ClientSession, user_id: str, **kwargs: Any) -> Any:
        return await cls._delete(session, f"{cls.url_fragment}/{user_id}/mfa", **kwargs)

    @classmethod
    async def disable_mfa(cls, session: aiohttp.ClientSession, **kwargs: Any) -> Any:
        return await cls._delete(session, f"{cls.url_fragment}/mfa", **kwargs)

    @classmethod
    async def retrieve_user_role(cls, session: aiohttp.ClientSession, **kwargs: Any) -> Any:
        return await cls._get(session, f"{cls.url_fragment}/role", **kwargs)

    @classmethod
    async def send_password_reset_request(cls, session: aiohttp.ClientSession, email: str, org_id: str,
                                          **kwargs: Any) -> Any:
        warnings.warn(
            f"{cls.send_password_reset_request.__qualname__} is deprecated. "
            f"Please use {Auths.request_password_reset_email.__qualname__} instead.", DeprecationWarning)
        data = {
            "Email": email,
            "Identifier": org_id,
        }

        return await cls._patch(session, f"{cls.url_fragment}/sendreset", data=data, **kwargs)

    @classmethod
    async def reset_password(cls, session: aiohttp.ClientSession, reset_token: str, new_password: str,
                             **kwargs: Any) -> Any:
        warnings.warn(f"{cls.reset_password.__qualname__} is deprecated and will be removed.", DeprecationWarning)

        data = {
            "ResetToken": reset_token,
            "Password": new_password,
        }

        return await cls._patch(session, f"{cls.url_fragment}/reset", data=data, **kwargs)

    @classmethod
    async def send_account_verification_code(cls, session: aiohttp.ClientSession, user_id: str, org_id: str,
                                             **kwargs: Any) -> Any:
        return await cls._post(session, f"{cls.url_fragment}/verificationCode/{user_id}/{org_id}", {}, **kwargs)

    @classmethod
    async def verify_user_account(cls, session: aiohttp.ClientSession, verification_code: str, user_id: str,
                                  **kwargs: Any) -> Any:
        data = {
            "VerificationCode": verification_code,
            "ID": user_id,
        }

        return await cls._post(session, f"{cls.url_fragment}/verify", data=data, **kwargs)

    @classmethod
    async def remove(cls, session: aiohttp.ClientSession, **kwargs: Any) -> Any:
        return await cls._delete(session, cls.url_fragment, **kwargs)

    @classmethod
    async def delete_measurements_by_userid(cls, session: aiohttp.ClientSession, user_id: str, **kwargs: Any) -> Any:
        warnings.warn(f"{cls.delete_measurements_by_userid.__qualname__} is deprecated and will be removed.",
                      DeprecationWarning)

        return await cls._delete(session, f"{cls.url_fragment}/{user_id}/measurements", **kwargs)

    @classmethod
    async def change_password(cls, session: aiohttp.ClientSession, org_id: str, email: str, password: str,
                              new_password: str, **kwargs: Any) -> Any:
        data = {
            "Identifier": org_id,
            "Email": email,
            "Password": password,
            "NewPassword": new_password,
        }

        return await cls._post(session, f"{cls.url_fragment}/changepassword", data=data, **kwargs)

    @classmethod
    async def renew_token(cls, session: aiohttp.ClientSession, license_key: str, **kwargs: Any) -> Any:
        warnings.warn(
            f"{cls.renew_token.__qualname__} is deprecated. Please use {Auths.renew_token.__qualname__} instead.",
            DeprecationWarning)

        data = {
            "Key": license_key,
        }

        return await cls._post(session, f"{cls.url_fragment}/auth/renew", data=data, **kwargs)

    @classmethod
    async def logout(cls, session: aiohttp.ClientSession, **kwargs: Any) -> Any:
        status, body = await cls._delete(session, f"{cls.url_fragment}/auth", **kwargs)

        if status < 400:
            Settings.user_token = ""
            Settings.user_refresh_token = ""

        return status, body



# Copyright (c) Nuralogix. All rights reserved. Licensed under the MIT license.
# See LICENSE.txt in the project root for license information

class Settings:
    rest_url = "https://api.deepaffex.ai"
    ws_url = "wss://api.deepaffex.ai"
    device_id = ""
    device_token = ""
    device_refresh_token = ""
    role_id = ""
    user_id = ""
    user_token = ""
    user_refresh_token = ""
